<?php

namespace Drupal\commerce_paybox_payment\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_paybox_payment\Controller\PaymentController;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_paybox_payment\SignatureChecker;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Paybox Commerce Payment Gateway.
 *
 * @CommercePaymentGateway(
 *   id = "paybox",
 *   label = @Translation("Paybox"),
 *   display_label = @Translation("Paybox"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_paybox_payment\PluginForm\RedirectPayboxForm",
 *   },
 *   payment_method_types = {"credit_card"},
 * )
 */
final class Paybox extends OffsitePaymentGatewayBase {

  private const CURRENCY_CODE_MAPPING = [
    '978' => 'EUR',
    '840' => 'USD',
    '826' => 'GBP',
    '032' => 'ARS',
    '040' => 'ATS',
    '036' => 'AUD',
    '056' => 'BEF',
    '986' => 'BRL',
    '124' => 'CAD',
    '756' => 'CHF',
    '280' => 'DEM',
    '208' => 'DKK',
    '246' => 'FIM',
    '300' => 'GRD',
    '392' => 'JPY',
    '116' => 'KHR',
    '410' => 'KRW',
    '484' => 'MXN',
    '578' => 'NOK',
    '554' => 'NZD',
    '752' => 'SEK',
    '702' => 'SGD',
    '792' => 'TRL',
    '901' => 'TWD',
  ];

  private const LANGUAGE_CODE_MAPPING = [
    'fr' => 'fra',
    'it' => 'ita',
    'sv' => 'swe',
    'en' => 'gbr',
    'de' => 'deu',
    'pt-pt' => 'prt',
    'es' => 'esp',
    'nl' => 'nld',
  ];

  /**
   * LanguageManager Service.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  private $languageManager;

  /**
   * Paybox constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager Service.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   Payment Type Manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   Payment Method Type Manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   Time Service.
   * @param \Drupal\Core\Language\LanguageManager $languageManager
   *   LanguageManager Service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    PaymentTypeManager $payment_type_manager,
    PaymentMethodTypeManager $payment_method_type_manager,
    TimeInterface $time,
    LanguageManager $languageManager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('language_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_id' => '',
      'rank' => '',
      'login' => '',
      'secret_key' => '',
      'server_url' => '',
      'currency_code' => '',
      'payment_diff' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site'),
      '#size' => 20,
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];
    $form['rank'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Rank'),
      '#size' => 20,
      '#default_value' => $this->configuration['rank'],
      '#required' => TRUE,
    ];
    $form['login'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Login'),
      '#size' => 20,
      '#default_value' => $this->configuration['login'],
      '#required' => TRUE,
    ];
    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('HMAC Secret key'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];
    $form['server_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The Paybox payment server URL'),
      '#description' => $this->t('The absolute URL to the Paybox payment server to process transactions.'),
      '#size' => 100,
      '#default_value' => $this->configuration['server_url'],
      '#required' => TRUE,
    ];
    $form['currency_code'] = [
      '#type' => 'select',
      '#title' => $this->t('Currency'),
      '#options' => self::CURRENCY_CODE_MAPPING,
      '#default_value' => $this->configuration['currency_code'],
    ];
    $form['payment_diff'] = [
      '#type' => 'textfield',
      '#attributes' => [
        ' type' => 'number',
      ],
      '#title' => $this->t('Deferred payment'),
      '#default_value' => $this->configuration['payment_diff'],
    ];

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['merchant_id'] = $values['merchant_id'];
    $this->configuration['rank'] = $values['rank'];
    $this->configuration['login'] = $values['login'];
    $this->configuration['secret_key'] = $values['secret_key'];
    $this->configuration['server_url'] = $values['server_url'];
    $this->configuration['currency_code'] = $values['currency_code'];
    $this->configuration['payment_diff'] = $values['payment_diff'];
  }

  /**
   * Construct Paybox request data.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   Order entity.
   * @param \Drupal\commerce_payment\Entity\PaymentInterface $payment
   *   Payment entity.
   *
   * @return array
   *   Request data build.
   */
  public function request(OrderInterface $order, PaymentInterface $payment) {
    $plugin_config = $this->getConfiguration();

    // Save payment to have reference to pass to bank site.
    $payment->save();

    $paymentAmount = (float) $payment->getAmount()->getNumber() * 100;
    $data = [
      'PBX_SITE' => $plugin_config['merchant_id'],
      'PBX_RANG' => $plugin_config['rank'],
      'PBX_IDENTIFIANT' => $plugin_config['login'],
      'PBX_TOTAL' => str_pad((string) $paymentAmount, 3, '0', STR_PAD_LEFT),
      'PBX_DEVISE' => $plugin_config['currency_code'],
      'PBX_CMD' => $payment->id(),
      'PBX_PORTEUR' => $order->getCustomer()->getEmail(),
      'PBX_RETOUR' => 'Mt:M;Ref:R;Transaction:S;Date:Q;Auto:A;Error:E;Signature:K',
      'PBX_HASH' => 'SHA512',
      'PBX_TIME' => date('c', $order->getCreatedTime()),
    ];

    $language = $this->languageManager->getCurrentLanguage()->getId();
    if (!empty(self::LANGUAGE_CODE_MAPPING[$language])) {
      $data['PBX_LANGUE'] = strtoupper(self::LANGUAGE_CODE_MAPPING[$language]);
    }

    if (!empty($plugin_config['payment_diff'])) {
      $data['PBX_DIFF'] = (int) $plugin_config['payment_diff'];
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $paymentId = $request->query->get('Ref');
    $payment = $this->entityTypeManager->getStorage('commerce_payment')->load($paymentId);

    if (!$payment instanceof PaymentInterface) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    if (1 !== SignatureChecker::checkSignature($request)) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    if ('completed' === $payment->getState()->getString()) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    $payment->setState('canceled');
    $payment->save();
    parent::onCancel($order, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $paymentId = $request->query->get('Ref');
    $transactionId = $request->query->get('Transaction');
    $transactionDate = $request->query->get('Date');
    $payment = $this->entityTypeManager->getStorage('commerce_payment')->load($paymentId);

    if (!$payment instanceof PaymentInterface) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    if (1 !== SignatureChecker::checkSignature($request)) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    if ('completed' === $payment->getState()->getString()) {
      throw new PaymentGatewayException('Payment cannot be found');
    }

    $returnAmount = $request->query->get('Mt');
    $paymentAmount = (float) $payment->getAmount()->getNumber() * 100;
    $paymentAmount = str_pad((string) $paymentAmount, 3, '0', STR_PAD_LEFT);
    $error = $request->query->get('Error');
    if ($returnAmount !== $paymentAmount) {
      throw new PaymentGatewayException('Charged amount not equal to order amount');
    }
    if (!empty($error) && PaymentController::EMPTY_ERROR !== $error  && PaymentController::PENDING_CODE !== $error) {
      throw new PaymentGatewayException('An error has occurred during payment');
    }

    if (PaymentController::PENDING_CODE === $error) {
      $payment->setState('pending');
      $payment->set('remote_id', $transactionId);
      $payment->save();
      parent::onReturn($order, $request);
      return;
    }

    $payment->setState('completed');
    $payment->set('remote_id', $transactionId);
    $payment->set('authorized', strtotime('today ' . $transactionDate));
    $payment->save();
    parent::onReturn($order, $request);
  }

}
